--[[
Name: LibNameplate-1.0
Author(s): Cyprias (cyprias@gmail.com)
Documentation: http://www.wowace.com/addons/libnameplate-1-0/pages/main/
SVN:  svn://svn.wowace.com/wow/libnameplate-1-0/mainline/trunk
Description: Alerts addons when a nameplate is shown or hidden. Has API to get info such as name, level, class, ect from the nameplate. LibNameplate tries to function with the default nameplates, Aloft, caelNamePlates and TidyPlates (buggy).
Dependencies: LibStub, CallbackHandler-1.0
]]

--[[
	Todo

	Notes
		- hooking plates OnEnter requires the plates enable mouse. But doing that stops the mouseover ID being available. 
]]

local MAJOR, MINOR = "LibNameplate-1.0", 11
if not LibStub then error(MAJOR .. " requires LibStub.") return end

local lib = LibStub:NewLibrary(MAJOR, MINOR)
if not lib then return end

lib.callbacks = lib.callbacks or LibStub("CallbackHandler-1.0"):New(lib)
if not lib.callbacks then	error(MAJOR .. " CallbackHandler-1.0.") return end

-- Globals
local _G					= _G
local tostring				= tostring
local WorldFrame			= WorldFrame
local select				= select
local UnitExists			= UnitExists
local UnitGUID				= UnitGUID
local UnitName				= UnitName
local pairs					= pairs
local table_insert			= table.insert
local GetMouseFocus			= GetMouseFocus
local GetNumRaidMembers		= GetNumRaidMembers
local GetNumPartyMembers	= GetNumPartyMembers
local UnitIsUnit			= UnitIsUnit
local GetRaidTargetIndex	= GetRaidTargetIndex
local print					= print
local table_getn			= table.getn
local _ --localized underscore so FindGlobals doesn't bug me about it.
local IsAddOnLoaded			= IsAddOnLoaded
local unpack = unpack


--Conts
local scanDelay		= 1 -- Scan for new nameplates every 1 seconds. We hook OnShow so this doesn't need to be done rapidly.
local updateDelay	= 1 -- Update nameplate info every 1 seconds. This checks if a nameplate has a raid icon then trys to find the GUID to that icon.


local DEBUG = false
--[===[@debug@
DEBUG = true
--@end-debug@]===]

lib.realPlate = lib.realPlate or {}
lib.fakePlate = lib.fakePlate or {}

local function CmdHandler()
	DEBUG = not DEBUG
end
_G.SlashCmdList["LIBNAMEPLATEDEBUG"] = CmdHandler
_G.SLASH_LIBNAMEPLATEDEBUG1 = "/lnbug"
local function debugPrint(...)
	if DEBUG then
		print(...)
	end
end
lib.debugPrint = debugPrint

lib.nameplates		= lib.nameplates or {}
lib.GUIDs			= lib.GUIDs or {}
lib.names			= lib.names or {}
lib.onShowHooks = lib.onShowHooks or {}
lib.onHideHooks = lib.onHideHooks or {}
lib.onUpdateHooks = lib.onUpdateHooks or {}
--~ lib.setAlphaHooks = lib.setAlphaHooks or {}
lib.healthOnValueChangedHooks = lib.healthOnValueChangedHooks or {}

lib.plateGUIDs = lib.plateGUIDs or {}
lib.isOnScreen = lib.isOnScreen or {}
lib.isOnUpdating = lib.isOnUpdating or {}


--Region locations
lib.name_region = lib.name_region or {}
lib.level_region = lib.level_region or {}
lib.boss_region = lib.boss_region or {}
lib.elite_region = lib.elite_region or {}
lib.threat_region = lib.threat_region or {}
lib.hightlight_region = lib.hightlight_region or {}
lib.raidicon_region = lib.raidicon_region or {}

--bar locations
lib.health_bar = lib.health_bar or {}
lib.cast_bar = lib.cast_bar or {}


lib.combatStatus = lib.combatStatus or {}

--[[
TidyPlate region names
	old: "threatGlow", "healthBorder", "castBorder", "castNostop", "spellIcon", "highlightTexture", "nameText", "levelText", "dangerSkull", "raidIcon", "eliteIcon"
	new: "threatGlow", "healthborder", "castborder", "castnostop", "spellicon", "highlight", "name", "level", "dangerskull", "raidicon", "eliteicon"
TidyPlate child names
	old: "healthBar", "castBar"
	new: "castbar", "healthbar"
	
TidyPlate 'frame.regions' names.
	castborder, castnostop, dangerskull, eliteicon, healthborder, highlight, level, name, raidicon, specialArt, specialText, specialText2, threatGlow, threatborder
TidyPlate 'frame.bars' names.
	cast (real), castbar, health (real), healthbar, 
]]
local regionNames = { "threatGlow", "healthBorder", "castBorder", "castNostop", "spellIcon", "highlightTexture", "nameText", "levelText", "dangerSkull", "raidIcon", "eliteIcon" }
local regionIndex = {}
for i, name in pairs(regionNames) do 
	regionIndex[name] = i
end

local barNames = { "healthBar", "castBar"}
local barIndex = {}
for i, name in pairs(barNames) do 
	barIndex[name] = i
end





--[[
	these always return nil
	frame:IsClampedToScreen(), frame:IsIgnoringDepth(), frame:IsMovable(), frame:IsToplevel(), frame:GetDepth(), frame:GetEffectiveDepth(), 

]]
local function IsNamePlateFrame(frame)
	if frame:GetName() then
--~ 		debugPrint("IsNamePlateFrame","No name!")
		return false
	end

	if frame:GetID() ~= 0 then
		return false
	end
	
	if frame:GetObjectType() ~= "Frame" then
--~ 		debugPrint("IsNamePlateFrame","Not a frame!")
		return false
	end
		
	if frame.extended or frame.aloftData then
		--Tidyplates = extended, Aloft = aloftData
		--They sometimes remove & replace the children so this needs to be checked first.
		return true
	end

	if frame.done then --caelNP
		return true
	end
	
	if frame:GetNumChildren() == 0 then
--~ 		debugPrint("IsNamePlateFrame","No children?!!", frame:GetName(), frame.extended and frame.extended.bars)
		return false
	end
		
	if frame:GetNumRegions() == 0 then
		return false
	end
	
	return true
end



--[===[@debug@
function LC_TEST()--		/run LC_TEST()
	local c = {WorldFrame:GetChildren()}
	local frame, cFrame
	for i = 1, table_getn(c) do
		frame = c[i]
		if frame:IsShown() then
			--frame.extended
			debugPrint(IsNamePlateFrame(frame), "ID:"..tostring(frame:GetID()), "C:"..tostring(frame:GetNumChildren()), "R:"..tostring(frame:GetNumRegions()))
			
		end
	end
end
--@end-debug@]===]

local function ScanWorldFrameChildren(n, ...)
    for i = 1, n do
        local frame = select(i, ...)
        if frame:IsShown() and IsNamePlateFrame(frame) then
            if not lib.nameplates[frame] then
				lib:NameplateFirstLoad(frame)
                lib:SetupNameplate(frame)                    
            end
        end    
    end
end


local lastChildren = 0
local function FindNameplates()
    local curChildren = WorldFrame:GetNumChildren()
    if curChildren ~= lastChildren then
        lastChildren = curChildren
        ScanWorldFrameChildren(curChildren, WorldFrame:GetChildren())
    end
end



---------------------------
-- Get region locations
----------------------------
function lib:GetNameRegion(frame)
	if frame.extended and frame.extended.regions and frame.extended.regions.name then --TidyPlates
		return frame.extended.regions.name
		
	elseif frame.aloftData and frame.aloftData.nameTextRegion then --Aloft
		return frame.aloftData.nameTextRegion
		
	elseif frame.oldname then --dNameplates
		return frame.oldname
	end
	
	return select(regionIndex.nameText, frame:GetRegions())
end

function lib:GetLevelRegion(frame)
	if frame.extended and frame.extended.regions and frame.extended.regions.level then--TidyPlates
		return frame.extended.regions.level
		
	elseif frame.aloftData and frame.aloftData.levelTextRegion then --Aloft
		return frame.aloftData.levelTextRegion
		
	elseif frame.level then --dNameplates
		return frame.level
	end
	
	return select(regionIndex.levelText, frame:GetRegions() )
end

function lib:GetBossRegion(frame)
	if frame.extended and frame.extended.regions and frame.extended.regions.dangerskull then --tidyPlates
		return frame.extended.regions.dangerskull
	elseif frame.aloftData and frame.aloftData.bossIconRegion then --aloft
		return frame.aloftData.bossIconRegion
	elseif frame.boss then --dNameplates
		return frame.boss
	end
	return select(regionIndex.dangerSkull, frame:GetRegions())
end

function lib:GetEliteRegion(frame)
	if frame.extended and frame.extended.regions and frame.extended.regions.eliteicon then --tidyPlates
		return frame.extended.regions.eliteicon
	elseif frame.aloftData and frame.aloftData.stateIconRegion then --aloft
		return frame.aloftData.stateIconRegion
	elseif frame.elite then --dNameplates
		return frame.elite
	end
	return  select(regionIndex.eliteIcon, frame:GetRegions())
end


function lib:GetThreatRegion(frame)
	if frame.extended and frame.extended.regions and frame.extended.regions.threatGlow then
		return frame.extended.regions.threatGlow
	elseif frame.aloftData and frame.aloftData.nativeGlowRegion then
		return frame.aloftData.nativeGlowRegion
	elseif frame.oldglow then --dNameplates
		return frame.oldglow
	end
	
	return select(regionIndex.threatGlow, frame:GetRegions() )
end


function lib:GetHightlightRegion(frame)
	if frame.extended then
		if frame.extended.regions then
			if frame.extended.regions.highlight then
				return frame.extended.regions.highlight
			elseif frame.extended.regions.highlightTexture then --old tidyplates
				return frame.extended.regions.highlightTexture
			end
		end
	elseif frame.aloftData and frame.aloftData.highlightRegion then
		return frame.aloftData.highlightRegion
	elseif frame.highlight then --dNameplates
		return frame.highlight
	end
	
	return select(regionIndex.highlightTexture, frame:GetRegions())
end


function lib:GetRaidIconRegion(frame)
	if frame.extended and frame.extended.regions and frame.extended.regions.raidicon then
		return frame.extended.regions.raidicon
	elseif frame.aloftData and frame.aloftData.raidIconRegion then
		return frame.aloftData.raidIconRegion
	end
	return select(regionIndex.raidIcon, frame:GetRegions())
end

----------------------
-- Get bar frames
----------------------
function lib:GetHealthBar(frame)
	if frame.extended and frame.extended.bars and frame.extended.bars.health then
		return frame.extended.bars.health
--~ 	elseif frame.aloftData then
		--Aloft changes the bar color. Our functions will have to use aloftData.originalHealthBarR
	elseif frame.healthOriginal then --dNameplates
		return frame.healthOriginal
	end
	return select(barIndex.healthBar, frame:GetChildren())
end


function lib:GetCastBar(frame)
	if frame.extended and frame.extended.bars and frame.extended.bars.castbar then
		return frame.extended.bars.castbar
	elseif frame.aloftData and frame.aloftData.castBar then
		return frame.aloftData.castBar
	end
	return select(barIndex.healthBar, frame:GetChildren())
end

------------------------------------------------------------------------------------------------------------------
local function HideMouseoverRegion(frame)																		--
-- If we move the camera angle while the mouse is over a plate, that plate won't hide the mouseover texture.	--
-- So if we're mousing over someone's feet and a plate has the mouseover texture visible, 						--
-- it fools our code into thinking we're mousing over that plate.												--
-- This can be recreated by placing the mouse over a nameplate then holding rightclick and moving the canera.	--
-- Our UpdateNameplateInfo will see that the texture is still visible but no mouseover ID is available. 		--
------------------------------------------------------------------------------------------------------------------

	local region = lib.hightlight_region[frame] or lib:GetHightlightRegion(frame)
	if region and region.Hide then
		region:Hide()
	end
end



local function RecycleNameplate(frame)
--~ 	debugPrint("RecycleNameplate", frame)

	lib.callbacks:Fire("LibNameplate_RecycleNameplate", lib.fakePlate[frame] or frame)
	
	if lib.plateGUIDs[frame] then
		lib.GUIDs[lib.plateGUIDs[frame]] = false
	end
	
	local plateName = lib:GetName(frame)
	
	if plateName and lib.names[plateName] then
		lib.names[plateName] = false
	end
	
	
	
	lib.plateGUIDs[frame] = false
	local fake = lib.fakePlate[frame]
	if fake then
		lib.realPlate[fake] = false
	end
	lib.fakePlate[frame] = false
end

local function FoundPlateGUID(frame, GUID, unitID)
	lib.plateGUIDs[frame] = GUID
	
	lib.GUIDs[GUID] = frame
	lib.callbacks:Fire("LibNameplate_FoundGUID", lib.fakePlate[frame] or frame, GUID, unitID)
end

local function GetMouseoverGUID(frame)
	local unitID = "mouseover"
	if UnitExists(unitID) then
		FoundPlateGUID(frame, UnitGUID(unitID), unitID)
	end
end

local function FindPlateWithRaidIcon(iconNum)
	for frame in pairs(lib.nameplates) do 
		if lib:IsMarked(frame) and lib:GetRaidIcon(frame) == iconNum then
			return frame
		end
	end
	return nil
end

local function FindGUIDByRaidIcon(frame, raidNum, from)
	local group, num = "", 0
	if GetNumRaidMembers() > 1 then
		group, num = "raid", GetNumRaidMembers()
	elseif GetNumPartyMembers() > 0 then
		group, num = "party", GetNumPartyMembers()
	else
		return
	end
	
	local unitID, targetID, targetIndex
	for i = 1, num do
		unitID = group..i;
		targetID = unitID.."target"
		if UnitExists(targetID) and not UnitIsUnit("target", targetID) then
			targetIndex = GetRaidTargetIndex(targetID)
			if targetIndex and targetIndex == raidNum then
				debugPrint("FindGUIDByRaidIcon", from, "Icon:"..tostring(raidNum), "unitID:"..tostring(targetID), "GUID:"..tostring(UnitGUID(targetID)))
				FoundPlateGUID(frame, UnitGUID(targetID), targetID)
				return
			end
		end
	end
end

local function UpdateNameplateInfo(frame)
	if lib:IsMouseover(frame) and not UnitExists("mouseover") then
		HideMouseoverRegion(frame)
	end

	if not lib.plateGUIDs[frame] then
		if lib:IsMouseover(frame) then
			GetMouseoverGUID(frame)
		elseif lib:IsMarked(frame) then
			local raidNum = lib:GetRaidIcon(frame)
			if raidNum and raidNum > 0 then
				FindGUIDByRaidIcon(frame, raidNum, "UpdateNameplateInfo")
			end
		end
	end

--~ 	debugPrint("Updated nameplate", lib:GetName(frame))
	frame.lnpLastUpdate = 0
end

function lib.OnNameplateShow(frame, ...)
	lib:SetupNameplate(frame)
end
local function ourOnShow(...) return lib.OnNameplateShow(...) end

function lib.OnNameplateHide(frame, ...)
	lib.isOnScreen[frame] = false
	lib.isOnUpdating[frame] = false
	
	RecycleNameplate(frame)
end
local function ourOnHide(...) return lib.OnNameplateHide(...) end

--[[
local function ourOnHide(self, ...)
	lib.isOnScreen[self] = false
	RecycleNameplate(self)
end
]]

--[[
function lib.callbacks:OnUsed(target, eventname)
end

function lib.callbacks:OnUnused(target, eventname)
end
]]


function lib.OnNameplateUpdate(frame, elapsed, ...)
	lib.isOnUpdating[frame] = true --to make sure our hooks don't break.
	if frame.lnpCheckForTarget then --Check on the first OnUpdate after the frame's shown.
		frame.lnpCheckForTarget = false
		if not lib.plateGUIDs[frame] and frame:IsShown() and ((frame:GetAlpha() == 1) and UnitExists("target")) then
			FoundPlateGUID(frame, UnitGUID("target"), "target")
		end
	end

	frame.lnpLastUpdate = (frame.lnpLastUpdate or 0) + elapsed
	if frame.lnpLastUpdate > updateDelay then
		UpdateNameplateInfo(frame)

		if not lib.fakePlate[frame] and frame.extended then
			lib.realPlate[frame.extended] = frame
			lib.fakePlate[frame] = frame.extended

			lib.callbacks:Fire("LibNameplate_RecycleNameplate", frame)--Hide real plate so addon unhook their stuff.
			lib.callbacks:Fire("LibNameplate_NewNameplate", lib.fakePlate[frame])
		end

		local inCombat = lib:IsInCombat(frame)
		
		if lib.combatStatus[frame] ~= inCombat then
			lib.combatStatus[frame] = inCombat
			lib.callbacks:Fire("LibNameplate_CombatChange", lib.fakePlate[frame] or frame, inCombat)
		end
		
	end
end
local function ourOnUpdate(...) return lib.OnNameplateUpdate(...) end



--[[
function lib.OnSetAlpha(frame, ...)
	local plateName = lib:GetName(frame)
	debugPrint("OnSetAlpha", plateName, ...)
end
local function ourSetAlpha(...) return lib.OnSetAlpha(...) end
]]


function lib.healthOnValueChanged(frame, ...)
	local plate = frame:GetParent()
	
	--strange, when a nameplate's not on screen, we still get HP changes. It's not relyable but might be of use somehow...
	if plate and plate:IsShown() then
		lib.callbacks:Fire("LibNameplate_HealthChange", frame, ...)
--~ 		local plateName = lib:GetName(plate)
--~ 		debugPrint("OnValueChanged", plateName, ...)
	end
end
local function ourHealthOnValueChanged(...) return lib.healthOnValueChanged(...) end

--[[
--~ --Castbars are only shown on your target's nameplate. No sense adding callbacks for this.
--~ --Castbar show
local testing = {}
lib.onCastbarShowHooks = lib.onCastbarShowHooks or {}
function lib.OnCastbarShow(self, ...)
	if not testing[self] then	
		if UnitExists("target") and (UnitCastingInfo("target") or UnitChannelInfo("target")) then
			testing[self] = true
			
			debugPrint("OnCastbarShow 2", UnitCastingInfo("target"), UnitChannelInfo("target"))
		end
	end
end
local function ourCastbarShow(...) return lib.OnCastbarShow(...) end

--~ --castbar hide
lib.onCastbarHideHooks = lib.onCastbarHideHooks or {}
function lib.OnCastbarHide(self, ...)
end
local function ourCastbarHide(...) return lib.OnCastbarHide(...) end
]]



function lib:SaveFrameRegions(frame)
	self.name_region[frame] = self:GetNameRegion(frame)
	self.level_region[frame]  = self:GetLevelRegion(frame)
	self.boss_region[frame]  = self:GetBossRegion(frame)
	self.elite_region[frame]  = self:GetEliteRegion(frame)
	self.threat_region[frame]  = self:GetThreatRegion(frame)
	self.hightlight_region[frame]  = self:GetHightlightRegion(frame)
	self.raidicon_region[frame]  = self:GetRaidIconRegion(frame)

end
--~ lib:GetHealthBar(frame)


function lib:SaveFrameBars(frame)
	self.health_bar[frame] =  self:GetHealthBar(frame)
	self.cast_bar[frame] =  self:GetCastBar(frame)

end

local testing = false

function lib:HookNameplate(frame)
	if frame:HasScript("OnHide") and not self.onHideHooks[frame] then
		self.onHideHooks[frame] = true
		frame:HookScript("OnHide", ourOnHide)
	end
	
	if frame:HasScript("OnShow") and not self.onShowHooks[frame] then
		self.onShowHooks[frame] = true
		frame:HookScript("OnShow", ourOnShow)
	end

	if frame:HasScript("OnUpdate") and not self.onUpdateHooks[frame] then
		self.onUpdateHooks[frame] = true
		frame:HookScript("OnUpdate", ourOnUpdate)
	end
	
	--[[if not self.setAlphaHooks[frame] then --nameRegion:HasScript("SetText") and 
		self.setAlphaHooks[frame] = true
		hooksecurefunc(frame, "SetAlpha", ourSetAlpha)
	end]]

	local healthBar = self.health_bar[frame]
	if healthBar and not self.healthOnValueChangedHooks[frame] then
		self.healthOnValueChangedHooks[frame] = true
		healthBar:HookScript("OnValueChanged", ourHealthOnValueChanged)
	end
end

function lib:NameplateFirstLoad(frame)
	--Save region and bar locations.
	lib:SaveFrameRegions(frame)
	lib:SaveFrameBars(frame)
	
	--Hook handlers.
	self:HookNameplate(frame)
	
	--Save frame's combat status as false.
	if self.combatStatus[frame] == nil then
		self.combatStatus[frame] = false --not in combat
	end
end

function lib:SetupNameplate(frame)
	self.isOnScreen[frame] = true --to make sure our hooks don't break.

	local plateName = self:GetName(frame)
	self.nameplates[frame] = plateName
	self.names[plateName] = frame
--~ 	local plateType = self:GetType(frame)
--~ 	local plateCombat = self:IsInCombat(frame)
--~ 	debugPrint("SetupNameplate", plateName, plateType, plateCombat)
	
	
	
	--TidyPlates replace the orginal frame with their own.
	--Lets save this and give that frame to addons. It's better for anchors.
	if frame.extended and not self.fakePlate[frame] then
		self.fakePlate[frame] = frame.extended
		self.realPlate[frame.extended] = frame
		
		--Without this was causing problems with PlateBuffs where it was never told the real plate was gone.
		self.callbacks:Fire("LibNameplate_RecycleNameplate", frame)
	end
	
	self.callbacks:Fire("LibNameplate_NewNameplate", self.fakePlate[frame] or frame)

	frame.lnpCheckForTarget = true
	UpdateNameplateInfo(frame)
end--	/lnbug

local function CheckForTargetGUID()
	local unitID = "target"
	local GUID
	for frame in pairs(lib.nameplates) do 
		if lib:IsTarget(frame) then
			lib.targeted = frame
			if not lib.plateGUIDs[frame] then
				FoundPlateGUID(frame, UnitGUID(unitID), unitID)
			end
			lib.callbacks:Fire("LibNameplate_TargetNameplate", lib.fakePlate[frame] or frame)
			return
		end
	end
--~ 	debugPrint("Failed to find target GUID")
end

local function OnUpdate(frame, elapsed)
	if frame.checkTarget then
		frame.checkTarget = false
		CheckForTargetGUID()
	end

	FindNameplates()

	--code searches for broken OnShow/OnHide hooks.
	--some nameplate addons will use SetScript instead of HookScript and breaks our hooks.
	frame.lastUpdate = frame.lastUpdate + elapsed
	if frame.lastUpdate >= scanDelay then
		frame.lastUpdate = 0
		for frame, value in pairs(lib.isOnScreen) do 
			if (value == true and not frame:IsShown()) then --OnHide fail
				debugPrint("OnHide fail", frame, value, frame:IsShown())
				lib.onHideHooks[frame] = false
				lib.isOnScreen[frame] = false
				lib:HookNameplate(frame)
				lib.OnNameplateHide(frame)
				
			elseif (value == false and frame:IsShown()) then --OnShow fail
				debugPrint("OnShow fail", frame, value, frame:IsShown())
				lib.onShowHooks[frame] = false
				lib.isOnScreen[frame] = false
				lib:HookNameplate(frame)
				lib:SetupNameplate(frame, true)
				
			end
		end
		for frame, value in pairs(lib.isOnUpdating) do 
			if value == false and frame:IsShown() then
				debugPrint("OnUpdate fail?")
				lib.onUpdateHooks[frame] = false
				lib:HookNameplate(frame)
			end
		end
	end
end


local function OnEvent(frame, event, ...)
	if event == "UPDATE_MOUSEOVER_UNIT" then
		if GetMouseFocus():GetName() == "WorldFrame" then
			local i = 0
			local mouseoverPlate
			for frame in pairs(lib.nameplates) do 
				if frame:IsShown() and lib:IsMouseover(frame) then
					i = i + 1
					mouseoverPlate = frame

				end
			end
			if i == 1 then
				if not lib.plateGUIDs[mouseoverPlate] then
					GetMouseoverGUID(mouseoverPlate)
				end
				lib.callbacks:Fire("LibNameplate_MouseoverNameplate", lib.fakePlate[mouseoverPlate] or mouseoverPlate)
			elseif i > 1 then
				debugPrint(i.." mouseover frames")
			end
			
		end
		
	elseif event == "PLAYER_TARGET_CHANGED" then
		if UnitExists("target") then
			frame.checkTarget = true--Check for target nameplate on next OnUpdate. We can't do it now.
		else
			lib.targeted = nil
		end
		
	elseif event == "UNIT_TARGET" then
		local unitID = ...
		local targetID = unitID.."target"
		if UnitExists(targetID) and not UnitIsUnit("player", unitID) then
			local iconNum = GetRaidTargetIndex(targetID)
			if iconNum and iconNum > 0 then
				local foundPlate = FindPlateWithRaidIcon(iconNum)
				if foundPlate and not lib.plateGUIDs[foundPlate] then
--~ 					debugPrint(event, "Found raid icon on ", UnitName(unitID), "'s target", UnitName(targetID), "icon:"..tostring(iconNum))
					FoundPlateGUID(foundPlate, UnitGUID(targetID), targetID)
				end
			end
		end
		
	elseif event == "RAID_TARGET_UPDATE" then
		for frame in pairs(lib.nameplates) do 
			if frame:IsShown() and not lib.plateGUIDs[frame] and lib:IsMarked(frame) then
				local raidNum = lib:GetRaidIcon(frame)
				if raidNum and raidNum > 0 then
					FindGUIDByRaidIcon(frame, raidNum, event)
				end
			end
		end
	elseif event == "PLAYER_ENTERING_WORLD" then
--~ 		debugPrint(event, MAJOR, _G.TidyPlates, _G.Aloft)
--~ 		if _G.TidyPlates then
--~ 			tidyPlatesRunning = true
--~ 		elseif _G.Aloft then
--~ 			aloftRunning = true
		if IsAddOnLoaded("caelNamePlates") then
			scanDelay = 0.1 --cael breaks our onhide/onshow hooks. 
		elseif IsAddOnLoaded("dNameplates") then
			scanDelay = 0.1 --cael breaks our onhide/onshow hooks. 
--~ 			debugPrint("dNameplates")
		end
		
	end
end

lib.frame = lib.frame or CreateFrame("Frame")

lib.frame.lastUpdate = 0
lib.frame.checkTarget = false
lib.frame:SetScript("OnUpdate", OnUpdate)
lib.frame:SetScript("OnEvent", OnEvent)
lib.frame:RegisterEvent('UPDATE_MOUSEOVER_UNIT')
lib.frame:RegisterEvent('PLAYER_TARGET_CHANGED')
lib.frame:RegisterEvent('UNIT_TARGET')
lib.frame:RegisterEvent('RAID_TARGET_UPDATE')
lib.frame:RegisterEvent('PLAYER_ENTERING_WORLD')